import JSZip from 'jszip';
import { ofetch } from 'ofetch';
import resolvePath from 'resolve-pathname';
import json5 from 'json5';

async function downloadVSCodeTheme(vscodeThemeId) {
  const [publisher, extId, theme] = vscodeThemeId.split(".");
  const themeLink = `https://${publisher}.gallery.vsassets.io/_apis/public/gallery/publisher/${publisher}/extension/${extId}/latest/assetbyname/Microsoft.VisualStudio.Services.VSIXPackage`;
  const buffer = await ofetch(themeLink, { responseType: "arrayBuffer" });
  const zip = await JSZip.loadAsync(buffer);
  const pkgContent = await zip.file("extension/package.json").async("string");
  const pkgJSON = JSON.parse(pkgContent);
  const themeConfig = (pkgJSON.contributes.themes || []).find(
    (t) => t.label.toLowerCase() === theme.toLowerCase()
  );
  if (!themeConfig) {
    const avaliableThemes = (pkgJSON.contributes.themes || []).map((t) => `\`${t.label}\``).join(" | ");
    throw new Error(`Not found theme \`${theme}\`, but found ${avaliableThemes}`);
  }
  const themePath = resolvePath(themeConfig.path, "extension/");
  const themeContent = await zip.file(themePath).async("string");
  const themeJSON = json5.parse(themeContent);
  return themeJSON;
}
function isDarkTheme(vscodeTheme) {
  if (vscodeTheme.type === "dark") {
    return true;
  }
  const background = vscodeTheme.colors?.["editor.background"];
  const foreground = vscodeTheme.colors?.["editor.foreground"];
  if (background && isDarkColor(background) || foreground && !isDarkColor(foreground)) {
    return true;
  }
  return false;
}
function isDarkColor(color) {
  const hex = normalizeHexColor(color).replace(/#/, "");
  const [r, g, b] = hex.match(/.{2}/g).map((hex2) => parseInt(hex2, 16));
  const brightness = (r * 299 + g * 587 + b * 114) / 1e3;
  return brightness <= 155;
}
function normalizeHexColor(color) {
  let hex = color.replace(/#/, "");
  if (hex.length < 6) {
    hex = hex.split("").map((char) => char + char).join("");
  }
  return "#" + hex;
}
function normalizeThemeName(name) {
  const hyphenateRE = /\B([A-Z\s])/g;
  return name.trim().replace(/\s/g, "-").replace(hyphenateRE, "-$1").toLowerCase();
}
function getThemeStyle(selectorScope, theme) {
  const styles = Object.entries(selectorScope).map(([selector, VSCodeScope]) => {
    const style = getVSCodeScopeStyle(VSCodeScope, theme);
    return { selector, style };
  }).filter((css) => css.style);
  const combinedStyles = styles.reduce((all, style) => {
    if (all[style.style]) {
      all[style.style] += `,
${style.selector}`;
    } else {
      all[style.style] = style.selector;
    }
    return all;
  }, {});
  const tokenCSS = Object.entries(combinedStyles).map(([style, selector]) => `${selector} ${style}`).join("\n");
  const global = getDefaultThemeSetting(theme);
  const wrapperCSS = `pre code {
  display: block;
  color: ${global.foreground};
  background: ${global.background};
}`;
  return wrapperCSS + "\n" + tokenCSS;
}
function getVSCodeScopeStyle(scope, theme) {
  const tokenMatchScores = theme.tokenColors.filter((token) => token.scope).map((token) => ({
    token,
    score: scoreScopeMatch(scope, token.scope)
  }));
  const maxMatchedToken = tokenMatchScores.reduce((all, cur) => cur.score > all.score ? cur : all, tokenMatchScores[0]);
  return maxMatchedToken.score ? getVSCodeTokenStyle(maxMatchedToken.token.settings) : null;
}
function scoreScopeMatch(sourceScope, targetScope) {
  if (Array.isArray(targetScope)) {
    const isExactlyMathched = targetScope.some((s) => s === sourceScope);
    if (isExactlyMathched) {
      return Infinity;
    } else {
      const scores = targetScope.map((s) => scoreScopeMatch(sourceScope, s));
      const maxScore = scores.reduce((all, cur) => Math.max(all, cur), 0);
      return maxScore;
    }
  } else if (!targetScope) {
    return 0;
  } else {
    if (sourceScope === targetScope) {
      return Infinity;
    } else {
      const sourceTokens = sourceScope.split(".");
      const targetTokens = targetScope.split(".");
      let score = 0;
      while (sourceTokens.length && targetTokens.length) {
        const sourceToken = sourceTokens.shift();
        const targetToken = targetTokens.shift();
        if (sourceToken === targetToken) {
          score += 1;
        } else {
          break;
        }
      }
      return score;
    }
  }
}
function getVSCodeTokenStyle(settings) {
  const mappings = [
    ["color", settings.foreground],
    ["background", settings.background],
    ["font-style", settings.fontStyle]
  ];
  const styles = mappings.filter((mapping) => mapping[1]).map((mapping) => `  ${mapping[0]}: ${mapping[1]};`).join("\n");
  return styles ? `{
${styles}
}` : null;
}
const VSCODE_FALLBACK_EDITOR_FG = { light: "#333333", dark: "#bbbbbb" };
const VSCODE_FALLBACK_EDITOR_BG = { light: "#fffffe", dark: "#1e1e1e" };
function getDefaultThemeSetting(theme) {
  const settings = theme.settings || theme.tokenColors;
  const globalSetting = settings?.find((s) => !s.scope);
  const foreground = parseDefaultColor("foreground");
  const background = parseDefaultColor("background");
  return { foreground, background };
  function parseDefaultColor(key) {
    const VSCODE_FALLBACK_SETTING = key === "foreground" ? VSCODE_FALLBACK_EDITOR_FG : VSCODE_FALLBACK_EDITOR_BG;
    const VSCODE_FALLBACK_COLOR = theme.type === "light" ? VSCODE_FALLBACK_SETTING.light : VSCODE_FALLBACK_SETTING.dark;
    return globalSetting?.settings?.[key] || theme.colors?.[`editor.${key}`] || VSCODE_FALLBACK_COLOR;
  }
}

export { VSCODE_FALLBACK_EDITOR_BG, VSCODE_FALLBACK_EDITOR_FG, downloadVSCodeTheme, getDefaultThemeSetting, getThemeStyle, getVSCodeScopeStyle, getVSCodeTokenStyle, isDarkColor, isDarkTheme, normalizeHexColor, normalizeThemeName, scoreScopeMatch };
